###
# #%L
# aiSSEMBLE::Test::MDA::Machine Learning::Inference
# %%
# Copyright (C) 2021 Booz Allen
# %%
# This software package is licensed under the Booz Allen Public License. All Rights Reserved.
# #L%
###
from abc import ABC
from .adversarial_threat_prediction_field import AdversarialThreatPredictionField
from typing import Any, Dict
import jsonpickle

class AdversarialThreatPredictionBase(ABC):
    """
    Base implementation of the Record for AdversarialThreatPrediction.

    GENERATED CODE - DO NOT MODIFY (add your customizations in AdversarialThreatPrediction).

    Generated from: templates/data-delivery-data-records/record.base.py.vm
    """

    def __init__(self):
        """
        Default constructor for this record.
        """
        self._score: int = None
        self._threat_detected: bool = None


    @classmethod
    def from_dict(cls, dict_obj: Dict[str, Any]):
        """
        Creates a record with the given dictionary's data.
        """
        record = cls()
        if dict_obj is not None:
            record.score = dict_obj.get('score')

            record.threat_detected = dict_obj.get('threatDetected')

        return record


    def as_dict(self) -> Dict[str, Any]:
        """
        Returns this record as a dictionary.
        """
        dict_obj = dict()

        dict_obj['score'] = self.score
        dict_obj['threatDetected'] = self.threat_detected

        return dict_obj


    @classmethod
    def from_json(cls, json_str: str):
        """
        Creates a record with the given json string's data.
        """
        dict_obj = jsonpickle.decode(json_str)
        return cls.from_dict(dict_obj)


    def as_json(self) -> str:
        """
        Returns this record as a json string.
        """
        return jsonpickle.encode(self.as_dict())


    @property
    def score(self) -> int:
        return self._score


    @score.setter
    def score(self, score: int) -> None:
        self._score = score


    @property
    def threat_detected(self) -> bool:
        return self._threat_detected


    @threat_detected.setter
    def threat_detected(self, threat_detected: bool) -> None:
        self._threat_detected = threat_detected


    def validate(self) -> None:
        """
        Performs the validation for this record.
        """
        self.validate_fields()


    def validate_fields(self) -> None:
        if self.score is None:
            raise ValueError('Field \'score\' is required')

        if self.threat_detected is None:
            raise ValueError('Field \'threatDetected\' is required')



    def get_value_by_field(self, field: AdversarialThreatPredictionField) -> any:
        """
        Returns the value of the given field for this record.
        """
        value = None
        if field == AdversarialThreatPredictionField.SCORE:
            value = self.score
        if field == AdversarialThreatPredictionField.THREAT_DETECTED:
            value = self.threat_detected

        return value



###
# #%L
# aiSSEMBLE::Test::MDA::Machine Learning::Inference
# %%
# Copyright (C) 2021 Booz Allen
# %%
# This software package is licensed under the Booz Allen Public License. All Rights Reserved.
# #L%
###
from abc import ABC


class DateAsSecondsSinceEpochBase(ABC):
    """
    Base implementation of the dateAsSecondsSinceEpoch dictionary type from BaseCyberTypesDictionary.

    GENERATED CODE - DO NOT MODIFY (add your customizations in DateAsSecondsSinceEpoch).

    Generated from: templates/data-delivery-data-records/dictionary.type.base.py.vm
    """

    MIN_VALUE: int = int('0')


    def __init__(self, value: int):
        if value is not None:
            self._value = int(value)
        else:
            self._value = None


    @property
    def value(self) -> int:
        return self._value


    @value.setter
    def value(self, value: int) -> None:
        if value is not None:
            value = int(value)
            self._value = value
        else:
            self._value = None


    def validate(self) -> None:
        """
        Performs the validation for this dictionary type.
        """
        self.validate_value()


    def validate_value(self) -> None:
        if (self._value is not None) and (self._value < DateAsSecondsSinceEpochBase.MIN_VALUE):
            raise ValueError('DateAsSecondsSinceEpoch value of \'%s\' is less than the minimum value of %s' % (self._value, DateAsSecondsSinceEpochBase.MIN_VALUE))



###
# #%L
# aiSSEMBLE::Test::MDA::Machine Learning::Inference
# %%
# Copyright (C) 2021 Booz Allen
# %%
# This software package is licensed under the Booz Allen Public License. All Rights Reserved.
# #L%
###
"""
    Base implementation of the message envelope for inference analytics.

    GENERATED CODE - DO NOT MODIFY (add your customizations in validation/inference_message_definition.py).

    Generated from: templates/inference/inference.message.base.py.vm
"""

from typing import Dict, List, Optional
from pydantic import BaseModel
from pandas import DataFrame

from ...validation.inference_payload_definition import Record, Inference

class RequestBodyBase(BaseModel):
    """
    Base class representing an inference request containing the data to use for the model prediction.
    """
    data: List[Record]

    def data_to_dict(self) -> Dict:
        """
        Convenience method to convert the data records into dictionaries.
        :return:
        """
        return [record.dict() for record in self.data]

    def prep_data(self) -> DataFrame:
        """
        Prep the data in the inference request for model prediction - override to
        provide the appropriate logic.
        :return:
        """

        return DataFrame(self.data_to_dict())


class ResponseBodyBase(BaseModel):
    """
    Base class representing a response containing the inference results of the model prediction.
    """
    inferences: List[Inference]



###
# #%L
# aiSSEMBLE::Test::MDA::Machine Learning::Inference
# %%
# Copyright (C) 2021 Booz Allen
# %%
# This software package is licensed under the Booz Allen Public License. All Rights Reserved.
# #L%
###
"""
    Base implementation of the request/response payloads for inference analytics.

    GENERATED CODE - DO NOT MODIFY (add your customizations in validation/inference_payload_definition.py).

    Generated from: templates/inference/inference.payload.base.py.vm
"""

from pydantic import BaseModel, validator
from ...dictionary.ip_address import IpAddress
from ...dictionary.event_kind import EventKind
from ...dictionary.event_category import EventCategory
from ...dictionary.event_outcome import EventOutcome
from ...dictionary.date_as_seconds_since_epoch import DateAsSecondsSinceEpoch

class RecordBase(BaseModel):
    """
    Represents a raw record in the inference request.
    """

    source_ip_address: str = None 

    @validator('source_ip_address')
    def validate_source_ip_address(cls, v):
        IpAddress(v).validate()
        return v

    created: int

    @validator('created')
    def validate_created(cls, v):
        DateAsSecondsSinceEpoch(v).validate()
        return v

    kind: str

    @validator('kind')
    def validate_kind(cls, v):
        EventKind(v).validate()
        return v

    category: str

    @validator('category')
    def validate_category(cls, v):
        EventCategory(v).validate()
        return v

    outcome: str

    @validator('outcome')
    def validate_outcome(cls, v):
        EventOutcome(v).validate()
        return v


class InferenceBase(BaseModel):
    """
    Represents an inference result of the model prediction.
    """

    score: int

    threat_detected: bool





# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: aissemble_machine_learning_inference/generated/inference/grpc/inference_api.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from aissemble_machine_learning_inference.generated.inference.grpc.generated import inference_payload_definition_pb2 as aissemble__machine__learning__inference_dot_generated_dot_inference_dot_grpc_dot_generated_dot_inference__payload__definition__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nQaissemble_machine_learning_inference/generated/inference/grpc/inference_api.proto\x1ajaissemble_machine_learning_inference/generated/inference/grpc/generated/inference_payload_definition.proto\")\n\x10InferenceRequest\x12\x15\n\x04\x64\x61ta\x18\x01 \x01(\x0b\x32\x07.Record\"/\n\x11InferenceResponse\x12\x1a\n\x06result\x18\x01 \x01(\x0b\x32\n.Inference\"B\n\x15\x42\x61tchInferenceRequest\x12\x12\n\nrow_id_key\x18\x01 \x01(\t\x12\x15\n\x04\x64\x61ta\x18\x02 \x03(\x0b\x32\x07.Record\"\xa9\x01\n\x16\x42\x61tchInferenceResponse\x12\x44\n\x07results\x18\x01 \x03(\x0b\x32\x33.BatchInferenceResponse.RecordRowIdAndInferencePair\x1aI\n\x1bRecordRowIdAndInferencePair\x12\x0e\n\x06row_id\x18\x01 \x01(\t\x12\x1a\n\x06result\x18\x02 \x01(\x0b\x32\n.Inference2\x89\x01\n\x10InferenceService\x12\x32\n\x07\x41nalyze\x12\x11.InferenceRequest\x1a\x12.InferenceResponse\"\x00\x12\x41\n\x0c\x41nalyzeBatch\x12\x16.BatchInferenceRequest\x1a\x17.BatchInferenceResponse\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'aissemble_machine_learning_inference.generated.inference.grpc.inference_api_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_INFERENCEREQUEST']._serialized_start=193
  _globals['_INFERENCEREQUEST']._serialized_end=234
  _globals['_INFERENCERESPONSE']._serialized_start=236
  _globals['_INFERENCERESPONSE']._serialized_end=283
  _globals['_BATCHINFERENCEREQUEST']._serialized_start=285
  _globals['_BATCHINFERENCEREQUEST']._serialized_end=351
  _globals['_BATCHINFERENCERESPONSE']._serialized_start=354
  _globals['_BATCHINFERENCERESPONSE']._serialized_end=523
  _globals['_BATCHINFERENCERESPONSE_RECORDROWIDANDINFERENCEPAIR']._serialized_start=450
  _globals['_BATCHINFERENCERESPONSE_RECORDROWIDANDINFERENCEPAIR']._serialized_end=523
  _globals['_INFERENCESERVICE']._serialized_start=526
  _globals['_INFERENCESERVICE']._serialized_end=663
# @@protoc_insertion_point(module_scope)



###
# #%L
# aiSSEMBLE::Test::MDA::Machine Learning::Inference
# %%
# Copyright (C) 2021 Booz Allen
# %%
# This software package is licensed under the Booz Allen Public License. All Rights Reserved.
# #L%
###
from abc import ABC
from re import fullmatch
from typing import List


class IpAddressBase(ABC):
    """
    Base implementation of the ipAddress dictionary type from BaseCyberTypesDictionary.

    GENERATED CODE - DO NOT MODIFY (add your customizations in IpAddress).

    Generated from: templates/data-delivery-data-records/dictionary.type.base.py.vm
    """

    FORMATS: List[str] = ['^((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$']


    def __init__(self, value: str):
        if value is not None:
            self._value = str(value)
        else:
            self._value = None


    @property
    def value(self) -> str:
        return self._value


    @value.setter
    def value(self, value: str) -> None:
        if value is not None:
            value = str(value)
            self._value = value
        else:
            self._value = None


    def validate(self) -> None:
        """
        Performs the validation for this dictionary type.
        """
        self.validate_format()


    def validate_format(self) -> None:
        if self._value and self._value.strip():
            validFormat = False
            for format in IpAddressBase.FORMATS:
                if fullmatch(format, self._value):
                    validFormat = True
                    break

            if not validFormat:
                raise ValueError('IpAddress value of \'%s\' did not match any valid formats' % self._value)



###
# #%L
# aiSSEMBLE::Test::MDA::Machine Learning::Inference
# %%
# Copyright (C) 2021 Booz Allen
# %%
# This software package is licensed under the Booz Allen Public License. All Rights Reserved.
# #L%
###
from abc import ABC
from ...dictionary.date_as_seconds_since_epoch import DateAsSecondsSinceEpoch
from ...dictionary.event_category import EventCategory
from ...dictionary.event_kind import EventKind
from ...dictionary.event_outcome import EventOutcome
from ...dictionary.ip_address import IpAddress
from .event_log_entry_field import EventLogEntryField
from typing import Any, Dict
import jsonpickle

class EventLogEntryBase(ABC):
    """
    Base implementation of the Record for EventLogEntry.

    GENERATED CODE - DO NOT MODIFY (add your customizations in EventLogEntry).

    Generated from: templates/data-delivery-data-records/record.base.py.vm
    """

    def __init__(self):
        """
        Default constructor for this record.
        """
        self._source_ip_address: IpAddress = None
        self._created: DateAsSecondsSinceEpoch = None
        self._kind: EventKind = None
        self._category: EventCategory = None
        self._outcome: EventOutcome = None


    @classmethod
    def from_dict(cls, dict_obj: Dict[str, Any]):
        """
        Creates a record with the given dictionary's data.
        """
        record = cls()
        if dict_obj is not None:
            if dict_obj.get('sourceIpAddress') is not None:
                record.source_ip_address = IpAddress(dict_obj.get('sourceIpAddress'))
            else:
                record.source_ip_address = None

            if dict_obj.get('created') is not None:
                record.created = DateAsSecondsSinceEpoch(dict_obj.get('created'))
            else:
                record.created = None

            if dict_obj.get('kind') is not None:
                record.kind = EventKind(dict_obj.get('kind'))
            else:
                record.kind = None

            if dict_obj.get('category') is not None:
                record.category = EventCategory(dict_obj.get('category'))
            else:
                record.category = None

            if dict_obj.get('outcome') is not None:
                record.outcome = EventOutcome(dict_obj.get('outcome'))
            else:
                record.outcome = None

        return record


    def as_dict(self) -> Dict[str, Any]:
        """
        Returns this record as a dictionary.
        """
        dict_obj = dict()

        if self.source_ip_address is not None:
            dict_obj['sourceIpAddress'] = self.source_ip_address.value
        else:
            dict_obj['sourceIpAddress'] = None
        if self.created is not None:
            dict_obj['created'] = self.created.value
        else:
            dict_obj['created'] = None
        if self.kind is not None:
            dict_obj['kind'] = self.kind.value
        else:
            dict_obj['kind'] = None
        if self.category is not None:
            dict_obj['category'] = self.category.value
        else:
            dict_obj['category'] = None
        if self.outcome is not None:
            dict_obj['outcome'] = self.outcome.value
        else:
            dict_obj['outcome'] = None

        return dict_obj


    @classmethod
    def from_json(cls, json_str: str):
        """
        Creates a record with the given json string's data.
        """
        dict_obj = jsonpickle.decode(json_str)
        return cls.from_dict(dict_obj)


    def as_json(self) -> str:
        """
        Returns this record as a json string.
        """
        return jsonpickle.encode(self.as_dict())


    @property
    def source_ip_address(self) -> IpAddress:
        return self._source_ip_address


    @source_ip_address.setter
    def source_ip_address(self, source_ip_address: IpAddress) -> None:
        self._source_ip_address = source_ip_address


    @property
    def created(self) -> DateAsSecondsSinceEpoch:
        return self._created


    @created.setter
    def created(self, created: DateAsSecondsSinceEpoch) -> None:
        self._created = created


    @property
    def kind(self) -> EventKind:
        return self._kind


    @kind.setter
    def kind(self, kind: EventKind) -> None:
        self._kind = kind


    @property
    def category(self) -> EventCategory:
        return self._category


    @category.setter
    def category(self, category: EventCategory) -> None:
        self._category = category


    @property
    def outcome(self) -> EventOutcome:
        return self._outcome


    @outcome.setter
    def outcome(self, outcome: EventOutcome) -> None:
        self._outcome = outcome


    def validate(self) -> None:
        """
        Performs the validation for this record.
        """
        self.validate_fields()


    def validate_fields(self) -> None:
        if self.source_ip_address is not None:
            self.source_ip_address.validate()

        if self.created is None:
            raise ValueError('Field \'created\' is required')
        else:
            self.created.validate()

        if self.kind is None:
            raise ValueError('Field \'kind\' is required')
        else:
            self.kind.validate()

        if self.category is None:
            raise ValueError('Field \'category\' is required')
        else:
            self.category.validate()

        if self.outcome is None:
            raise ValueError('Field \'outcome\' is required')
        else:
            self.outcome.validate()



    def get_value_by_field(self, field: EventLogEntryField) -> any:
        """
        Returns the value of the given field for this record.
        """
        value = None
        if field == EventLogEntryField.SOURCE_IP_ADDRESS:
            value = self.source_ip_address.value if self.source_ip_address is not None else None
        if field == EventLogEntryField.CREATED:
            value = self.created.value if self.created is not None else None
        if field == EventLogEntryField.KIND:
            value = self.kind.value if self.kind is not None else None
        if field == EventLogEntryField.CATEGORY:
            value = self.category.value if self.category is not None else None
        if field == EventLogEntryField.OUTCOME:
            value = self.outcome.value if self.outcome is not None else None

        return value



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from aissemble_machine_learning_inference.generated.inference.grpc import inference_api_pb2 as aissemble__machine__learning__inference_dot_generated_dot_inference_dot_grpc_dot_inference__api__pb2


class InferenceServiceStub(object):
    """Defines an inference analytic service
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Analyze = channel.unary_unary(
                '/InferenceService/Analyze',
                request_serializer=aissemble__machine__learning__inference_dot_generated_dot_inference_dot_grpc_dot_inference__api__pb2.InferenceRequest.SerializeToString,
                response_deserializer=aissemble__machine__learning__inference_dot_generated_dot_inference_dot_grpc_dot_inference__api__pb2.InferenceResponse.FromString,
                )
        self.AnalyzeBatch = channel.unary_unary(
                '/InferenceService/AnalyzeBatch',
                request_serializer=aissemble__machine__learning__inference_dot_generated_dot_inference_dot_grpc_dot_inference__api__pb2.BatchInferenceRequest.SerializeToString,
                response_deserializer=aissemble__machine__learning__inference_dot_generated_dot_inference_dot_grpc_dot_inference__api__pb2.BatchInferenceResponse.FromString,
                )


class InferenceServiceServicer(object):
    """Defines an inference analytic service
    """

    def Analyze(self, request, context):
        """Perform the relevant inference anayltic against a single input data record
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AnalyzeBatch(self, request, context):
        """Perform the relevant inference analytic against multiple input data records
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_InferenceServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Analyze': grpc.unary_unary_rpc_method_handler(
                    servicer.Analyze,
                    request_deserializer=aissemble__machine__learning__inference_dot_generated_dot_inference_dot_grpc_dot_inference__api__pb2.InferenceRequest.FromString,
                    response_serializer=aissemble__machine__learning__inference_dot_generated_dot_inference_dot_grpc_dot_inference__api__pb2.InferenceResponse.SerializeToString,
            ),
            'AnalyzeBatch': grpc.unary_unary_rpc_method_handler(
                    servicer.AnalyzeBatch,
                    request_deserializer=aissemble__machine__learning__inference_dot_generated_dot_inference_dot_grpc_dot_inference__api__pb2.BatchInferenceRequest.FromString,
                    response_serializer=aissemble__machine__learning__inference_dot_generated_dot_inference_dot_grpc_dot_inference__api__pb2.BatchInferenceResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'InferenceService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class InferenceService(object):
    """Defines an inference analytic service
    """

    @staticmethod
    def Analyze(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/InferenceService/Analyze',
            aissemble__machine__learning__inference_dot_generated_dot_inference_dot_grpc_dot_inference__api__pb2.InferenceRequest.SerializeToString,
            aissemble__machine__learning__inference_dot_generated_dot_inference_dot_grpc_dot_inference__api__pb2.InferenceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AnalyzeBatch(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/InferenceService/AnalyzeBatch',
            aissemble__machine__learning__inference_dot_generated_dot_inference_dot_grpc_dot_inference__api__pb2.BatchInferenceRequest.SerializeToString,
            aissemble__machine__learning__inference_dot_generated_dot_inference_dot_grpc_dot_inference__api__pb2.BatchInferenceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



###
# #%L
# aiSSEMBLE::Test::MDA::Machine Learning::Inference
# %%
# Copyright (C) 2021 Booz Allen
# %%
# This software package is licensed under the Booz Allen Public License. All Rights Reserved.
# #L%
###
from __future__ import annotations
from enum import Enum
from typing import List

class AdversarialThreatPredictionField(Enum):
    """
    Enum to represent the fields of a AdversarialThreatPrediction record.

    GENERATED CODE - DO NOT MODIFY

    Originally generated from: templates/data-delivery-data-records/record.field.enum.py.vm
    """

    SCORE = 0, None, None, None
    THREAT_DETECTED = 1, None, None, None


    def __new__(cls, index: int, drift_policy: str, ethics_policy: str, protection_policy: str):
        field = object.__new__(cls)
        field.index = index
        field._drift_policy = drift_policy
        field._ethics_policy = ethics_policy
        field._protection_policy = protection_policy
        return field


    @property
    def drift_policy(self) -> str:
        return self._drift_policy


    def has_drift_policy(self) -> bool:
        return True if self._drift_policy and self._drift_policy.strip() else False


    @property
    def ethics_policy(self) -> str:
        return self._ethics_policy


    def has_ethics_policy(self) -> bool:
        return True if self._ethics_policy and self._ethics_policy.strip() else False


    @property
    def protection_policy(self) -> str:
        return self._protection_policy


    def has_protection_policy(self) -> bool:
        return True if self._protection_policy and self._protection_policy.strip() else False


    @classmethod
    def get_fields_with_drift_policy(cls) -> List[AdversarialThreatPredictionField]:
        """
        Returns the list of fields that have a drift policy.
        """
        fields_with_drift_policy = []
        for field in AdversarialThreatPredictionField:
            if field.has_drift_policy():
                fields_with_drift_policy.append(field)

        return fields_with_drift_policy

    @classmethod
    def get_fields_with_ethics_policy(cls) -> List[AdversarialThreatPredictionField]:
        """
        Returns the list of fields that have an ethics policy.
        """
        fields_with_ethics_policy = []
        for field in AdversarialThreatPredictionField:
            if field.has_ethics_policy():
                fields_with_ethics_policy.append(field)

        return fields_with_ethics_policy

    @classmethod
    def get_fields_with_protection_policy(cls) -> List[AdversarialThreatPredictionField]:
        """
        Returns the list of fields that have a protection policy.
        """
        fields_with_protection_policy = []
        for field in AdversarialThreatPredictionField:
            if field.has_protection_policy():
                fields_with_protection_policy.append(field)

        return fields_with_protection_policy



###
# #%L
# aiSSEMBLE::Test::MDA::Machine Learning::Inference
# %%
# Copyright (C) 2021 Booz Allen
# %%
# This software package is licensed under the Booz Allen Public License. All Rights Reserved.
# #L%
###
from google.protobuf import json_format
from pydantic.error_wrappers import ValidationError
from grpc import StatusCode
import logging

from ....inference_impl import *
from .inference_api_pb2_grpc import InferenceServiceServicer

from .inference_api_pb2 import InferenceResponse as GrpcInferenceResponse, \
    BatchInferenceResponse as GrpcBatchInferenceResponse
from .generated.inference_payload_definition_pb2 import Inference as GrpcInference
from ....validation.inference_payload_definition import Record
from ....validation.inference_message_definition import RequestBody
from ....validation.inference_payload_definition import Inference as InferenceImplResult

# Utilized to support Pydantic-based validation of batch requests
from ..rest.inference_payload_definition import BatchInferenceRequest as RestBatchInferenceRequest

class InferenceGrpcBase(InferenceServiceServicer):
    """
    Base implementation of a gRPC endpoint for an inference analytic.

    GENERATED CODE - DO NOT MODIFY (add your inference analytic implementation to inference_impl.py).

    Generated from: templates/inference/inference.api.grpc.base.py.vm
    """
    def Analyze(self, request, context):
        """
        gRPC implementation for performing inferencing against a single provided input data record.
        :param request:
        :param context:
        :return:
        """
        infer_impl_data_record = self.validate_analyze_req_and_create_impl_record(request, context)

        infer_impl_request = RequestBody(data=[infer_impl_data_record])
        infer_impl_response = execute_inference(infer_impl_request)

        grpc_inference_result = self.convert_inference_impl_result_to_grpc_payload(infer_impl_response.inferences[0])
        return GrpcInferenceResponse(result=grpc_inference_result)

    def AnalyzeBatch(self, request, context):
        """
        gRPC implementation for performing inferencing against a batch of multiple provided input data records.
        :param request:
        :param context:
        :return:
        """
        infer_impl_data_records = self.validate_analyze_batch_req_and_create_impl_records(request, context)

        infer_impl_request = RequestBody(data=infer_impl_data_records)
        infer_impl_response = execute_inference(infer_impl_request)

        record_row_id_and_inference_pairs = []
        for index, inference_result in enumerate(infer_impl_response.inferences):
            record_row_id = getattr(request.data[index], request.row_id_key)

            record_row_id_and_inference_pair = GrpcBatchInferenceResponse.RecordRowIdAndInferencePair(
                row_id=record_row_id, result=self.convert_inference_impl_result_to_grpc_payload(inference_result))
            record_row_id_and_inference_pairs.append(record_row_id_and_inference_pair)

        return GrpcBatchInferenceResponse(results=record_row_id_and_inference_pairs)

    def convert_inference_impl_result_to_grpc_payload(self, inference_impl_result: InferenceImplResult):
        """
        Due to the lack of inheritance support with Protobuf defintions, this method provides an
        extension point for developers to convert the inference implementation result into its
        corresponding gRPC protobuf payload.  Typically, the default implementation of this method
        will suffice if the generated inference_payload_definition.proto is used as is.  However
        if developers need a hand-build a custom inference_payload_definition.proto, this method
        should be appropriately extended as needed.

        :param inference_impl_result: Inference result returned by the appropriate inference analytic
                    implementation.
        :return: given inference result converted to the appropriate gRPC result payload object.
        """
        return json_format.ParseDict(inference_impl_result.dict(), GrpcInference())

    def validate_analyze_req_and_create_impl_record(self, grpc_request, context):
        """
        Validates the given analyze request and creates the corresponding Record that will be
        provided to the relevant inference implementation.

        :param grpc_request:
        :param context:
        :return:
        """
        try:
            infer_impl_data_record = Record.parse_obj(
                json_format.MessageToDict(grpc_request.data, preserving_proto_field_name=True))
        except ValidationError as e:
            logging.warning(f"Invalid format exception: {str(e)}")
            context.abort(StatusCode.INVALID_ARGUMENT, str(e))
        return infer_impl_data_record

    def validate_analyze_batch_req_and_create_impl_records(self, grpc_request, context):
        """
        Validates the given analyze batch request and creates the corresponding list of Records that will be
        provided to the relevant inference implementation.

        :param grpc_request:
        :param context:
        :return:
        """
        grpc_data_record_dicts = []
        for grpc_data_record in grpc_request.data:
            grpc_data_record_dicts.append(json_format.MessageToDict(grpc_data_record, preserving_proto_field_name=True))

        try:
            rest_batch_infer_request = RestBatchInferenceRequest(row_id_key='unused', data=grpc_data_record_dicts)
        except ValidationError as e:
            logging.warning(f"Invalid format exception: {str(e)}")
            context.abort(StatusCode.INVALID_ARGUMENT, str(e))

        return rest_batch_infer_request.data



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: aissemble_machine_learning_inference/generated/inference/grpc/generated/inference_payload_definition.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\njaissemble_machine_learning_inference/generated/inference/grpc/generated/inference_payload_definition.proto\"e\n\x06Record\x12\x19\n\x11source_ip_address\x18\x01 \x01(\t\x12\x0f\n\x07\x63reated\x18\x02 \x01(\x05\x12\x0c\n\x04kind\x18\x03 \x01(\t\x12\x10\n\x08\x63\x61tegory\x18\x04 \x01(\t\x12\x0f\n\x07outcome\x18\x05 \x01(\t\"3\n\tInference\x12\r\n\x05score\x18\x01 \x01(\x05\x12\x17\n\x0fthreat_detected\x18\x02 \x01(\x08\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'aissemble_machine_learning_inference.generated.inference.grpc.generated.inference_payload_definition_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_RECORD']._serialized_start=110
  _globals['_RECORD']._serialized_end=211
  _globals['_INFERENCE']._serialized_start=213
  _globals['_INFERENCE']._serialized_end=264
# @@protoc_insertion_point(module_scope)



###
# #%L
# aiSSEMBLE::Test::MDA::Machine Learning::Inference
# %%
# Copyright (C) 2021 Booz Allen
# %%
# This software package is licensed under the Booz Allen Public License. All Rights Reserved.
# #L%
###



###
# #%L
# aiSSEMBLE::Test::MDA::Machine Learning::Inference
# %%
# Copyright (C) 2021 Booz Allen
# %%
# This software package is licensed under the Booz Allen Public License. All Rights Reserved.
# #L%
###
"""
Models the request/response message envelopes utilized to expose inference analytics as a
REST API. The classes defined here should align with the message envelopes defined in the corresponding
inference_api.proto definition, which supports concurrently exposing inference analyics as a
gRPC API.

GENERATED CODE - DO NOT MODIFY (specify inference analytic inputs/outputs in validation/inference_payload_definition.py).

Generated from: templates/inference/inference.payload.rest.py.vm
"""

from typing import List

from pydantic import BaseModel

from ....validation.inference_payload_definition import Record, Inference


class InferenceRequest(Record):
    """
    Defines the required request message elements needed to perform inferencing against a single input data record.
    """
    pass


class InferenceResponse(Inference):
    """
    Defines the response message containing the results of performing inferencing against a single data record.
    """
    pass


class RecordRowIdAndInferenceResultPair(BaseModel):
    """
    Helper transfer object container that links an inference result with the ID of its corresponding input data record.
    """
    row_id: str
    result: Inference


class BatchInferenceRequest(BaseModel):
    """
    Defines the required request message elements needed to perform inferencing against a batch of multiple
    input data records.  The row_id_key is utilized to specify the attribute within the given input data records
    that should be embedded alongside inference results to allow invoking clients to understand to which data record
    the inference result correlates.
    """
    row_id_key: str
    data: List[Record]


class BatchInferenceResponse(BaseModel):
    """
    Defines the response message containing the results of performing inferencing against the provided batch of multiple
    data records.  Each inference result is linked to its corresponding input data record via the row_id attribute value,
    which aligns with the attribute from the original input data record that was specified in the BatchInferenceRequest.
    """
    results: List[RecordRowIdAndInferenceResultPair]


